﻿/*
    Data generated using a custom made by Tom Grobbe.
    Data source: GTA V <update>_overlays.xml files.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace vMenuClient
{
    public enum TattooZone
    {
        ZONE_TORSO = 0,
        ZONE_HEAD = 1,
        ZONE_LEFT_ARM = 2,
        ZONE_RIGHT_ARM = 3,
        ZONE_LEFT_LEG = 4,
        ZONE_RIGHT_LEG = 5,
        ZONE_UNKNOWN = 6,
        ZONE_NONE = 7
    }

    public struct Tattoo
    {
        public int gender;
        public string name;
        public string collectionName;
        public TattooZone zoneId;
        public string type;

        /// <summary>
        /// Creates a new Tattoo object.
        /// </summary>
        /// <param name="gender">0 = male, 1 = female, 2 = both</param>
        /// <param name="name">Tattoo name</param>
        /// <param name="collectionName">Tattoo collection name</param>
        /// <param name="zoneId">Tattoo zone <see cref="TattooZone"/></param>
        /// <param name="type">Tattoo type</param>
        public Tattoo(int gender, string name, string collectionName, TattooZone zoneId, string type)
        {
            this.gender = gender;
            this.name = name;
            this.collectionName = collectionName;
            this.zoneId = zoneId;
            this.type = type;
        }
    }

    internal static class MaleTattoosCollection
    {
        internal static List<Tattoo> TORSO = new List<Tattoo>();
        internal static List<Tattoo> HEAD = new List<Tattoo>();
        internal static List<Tattoo> LEFT_ARM = new List<Tattoo>();
        internal static List<Tattoo> RIGHT_ARM = new List<Tattoo>();
        internal static List<Tattoo> LEFT_LEG = new List<Tattoo>();
        internal static List<Tattoo> RIGHT_LEG = new List<Tattoo>();
        internal static List<Tattoo> BADGES = new List<Tattoo>();
    }

    internal struct FemaleTattoosCollection
    {
        internal static List<Tattoo> TORSO = new List<Tattoo>();
        internal static List<Tattoo> HEAD = new List<Tattoo>();
        internal static List<Tattoo> LEFT_ARM = new List<Tattoo>();
        internal static List<Tattoo> RIGHT_ARM = new List<Tattoo>();
        internal static List<Tattoo> LEFT_LEG = new List<Tattoo>();
        internal static List<Tattoo> RIGHT_LEG = new List<Tattoo>();
        internal static List<Tattoo> BADGES = new List<Tattoo>();
    }

    internal static class TattoosData
    {
        private static bool isDataSetup = false;
        internal static void GenerateTattoosData()
        {
            if (isDataSetup)
            {
                return;
            }

            isDataSetup = true;

            foreach (var tattoo in Newtonsoft.Json.JsonConvert.DeserializeObject<List<Tattoo>>(Properties.Resources.overlays))
            {
                if (!string.IsNullOrEmpty(tattoo.name))
                {
                    if (tattoo.type == "TYPE_TATTOO" && !tattoo.name.ToLower().Contains("hair_"))
                    {
                        switch (tattoo.zoneId)
                        {
                            case TattooZone.ZONE_TORSO:
                                if (tattoo.gender == 0 || tattoo.gender == 2)
                                {
                                    MaleTattoosCollection.TORSO.Add(tattoo);
                                }
                                if (tattoo.gender == 1 || tattoo.gender == 2)
                                {
                                    FemaleTattoosCollection.TORSO.Add(tattoo);
                                }
                                break;
                            case TattooZone.ZONE_HEAD:
                                if (tattoo.gender == 0 || tattoo.gender == 2)
                                {
                                    MaleTattoosCollection.HEAD.Add(tattoo);
                                }
                                if (tattoo.gender == 1 || tattoo.gender == 2)
                                {
                                    FemaleTattoosCollection.HEAD.Add(tattoo);
                                }
                                break;
                            case TattooZone.ZONE_LEFT_ARM:
                                if (tattoo.gender == 0 || tattoo.gender == 2)
                                {
                                    MaleTattoosCollection.LEFT_ARM.Add(tattoo);
                                }
                                if (tattoo.gender == 1 || tattoo.gender == 2)
                                {
                                    FemaleTattoosCollection.LEFT_ARM.Add(tattoo);
                                }
                                break;
                            case TattooZone.ZONE_RIGHT_ARM:
                                if (tattoo.gender == 0 || tattoo.gender == 2)
                                {
                                    MaleTattoosCollection.RIGHT_ARM.Add(tattoo);
                                }
                                if (tattoo.gender == 1 || tattoo.gender == 2)
                                {
                                    FemaleTattoosCollection.RIGHT_ARM.Add(tattoo);
                                }
                                break;
                            case TattooZone.ZONE_LEFT_LEG:
                                if (tattoo.gender == 0 || tattoo.gender == 2)
                                {
                                    MaleTattoosCollection.LEFT_LEG.Add(tattoo);
                                }
                                if (tattoo.gender == 1 || tattoo.gender == 2)
                                {
                                    FemaleTattoosCollection.LEFT_LEG.Add(tattoo);
                                }
                                break;
                            case TattooZone.ZONE_RIGHT_LEG:
                                if (tattoo.gender == 0 || tattoo.gender == 2)
                                {
                                    MaleTattoosCollection.RIGHT_LEG.Add(tattoo);
                                }
                                if (tattoo.gender == 1 || tattoo.gender == 2)
                                {
                                    FemaleTattoosCollection.RIGHT_LEG.Add(tattoo);
                                }
                                break;
                            default:
                                break;
                        }
                    }
                    else if (tattoo.type == "TYPE_BADGE" && !tattoo.name.ToLower().Contains("hair_"))
                    {
                        if (tattoo.gender == 0 || tattoo.gender == 2)
                        {
                            MaleTattoosCollection.BADGES.Add(tattoo);
                        }
                        if (tattoo.gender == 1 || tattoo.gender == 2)
                        {
                            FemaleTattoosCollection.BADGES.Add(tattoo);
                        }
                    }
                    else if (tattoo.name.ToLower().Contains("hair_"))
                    {
                        if (tattoo.gender == 0 || tattoo.gender == 2)
                        {

                        }
                        if (tattoo.gender == 1 || tattoo.gender == 2)
                        {

                        }
                    }
                }
            }

        }

        /*
        internal static FemaleTattoosCollection FemaleTattoos = new FemaleTattoosCollection()
        {
            TORSO = new List<Tattoo>()
            {
                new Tattoo() { collectionName = "mpairraces_overlays", name = "MP_Airraces_Tattoo_000_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpairraces_overlays", name = "MP_Airraces_Tattoo_001_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpairraces_overlays", name = "MP_Airraces_Tattoo_002_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpairraces_overlays", name = "MP_Airraces_Tattoo_004_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpairraces_overlays", name = "MP_Airraces_Tattoo_005_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpairraces_overlays", name = "MP_Airraces_Tattoo_006_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpairraces_overlays", name = "MP_Airraces_Tattoo_007_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_000_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_001_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_003_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_005_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_006_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_008_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_010_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_011_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_013_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_017_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_018_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_019_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_021_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_023_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_026_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_029_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_030_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_031_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_032_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_034_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_039_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_041_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_043_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_050_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_052_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_058_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_059_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_060_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_005", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_006", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_009", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_011", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_013", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_015", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_016", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_017", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_018", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_019", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_028", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_000_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_002_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_003_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_005_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_008_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_009_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_010_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_011_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_015_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_016_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_019_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_020_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_021_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_022_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_024_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_026_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_027_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_000_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_001_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_009_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_010_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_012_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_013_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_014_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_017_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_018_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_019_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_020_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_022_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_028_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_029_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphalloween_overlays", name = "MP_Airraces_Tattoo_000_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphalloween_overlays", name = "MP_Airraces_Tattoo_001_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphalloween_overlays", name = "MP_Airraces_Tattoo_002_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphalloween_overlays", name = "MP_Airraces_Tattoo_004_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphalloween_overlays", name = "MP_Airraces_Tattoo_005_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphalloween_overlays", name = "MP_Airraces_Tattoo_006_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphalloween_overlays", name = "MP_Airraces_Tattoo_007_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_000", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_002", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_006", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_011", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_012", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_013", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_024", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_025", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_029", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_030", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_031", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_032", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_033", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_035", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_041", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_046", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_047", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_000_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_001_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_002_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_009_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_010_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_011_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_001_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_002_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_004_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_009_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_010_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_013_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_014_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_021_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_026_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_000_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_008_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_011_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_012_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_016_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_019_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_031_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_032_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_003_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_006_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_007_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_008_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_014_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_015_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_024_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_002_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_012_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_022_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_025_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_027_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_029_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_000_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_002_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_003_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_006_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_007_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_009_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_010_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_013_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_015_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_016_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_017_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_018_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_019_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_021_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_022_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_024_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_025_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_011_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_012_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_014_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_018_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_019_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_024_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_026_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_027_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_029_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_030_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_033_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_034_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_037_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_040_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_041_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_044_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_046_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_048_F", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_003", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_004", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_005", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_008", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_011", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_012", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_013", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_014", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_016", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_017", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_018", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_019", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_004", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_009", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_010", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_011", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_012", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_013", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_016", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_019", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_020", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_024", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_025", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_029", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_030", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_034", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_036", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_044", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_045", gender = 1, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_046", gender = 1, zoneId = TattooZone.ZONE_TORSO},
            },
            HEAD = new List<Tattoo>()
            {
                //new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_Biker_Hair_000_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_Biker_Hair_001_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_Biker_Hair_002_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_Biker_Hair_003_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_Biker_Hair_004_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_Biker_Hair_005_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_Biker_Hair_006_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_009_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_038_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_051_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_007", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_024", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_025", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_029", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Hair_F_000_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Hair_F_001_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_003_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Hair_000_a", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Hair_000_b", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Hair_000_c", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Hair_000_d", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Hair_000_e", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_005", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_021", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_F_Hair_017_a", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_F_Hair_017_b", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_F_Hair_017_c", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_F_Hair_017_d", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_F_Hair_017_e", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_F_Hair_020_a", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_F_Hair_020_b", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_F_Hair_020_c", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_F_Hair_020_d", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_F_Hair_020_e", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mplowrider_overlays", name = "LR_F_Hair_000", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mplowrider_overlays", name = "LR_F_Hair_001", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mplowrider_overlays", name = "LR_F_Hair_002", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mplowrider2_overlays", name = "LR_F_Hair_003", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mplowrider2_overlays", name = "LR_F_Hair_004", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mplowrider2_overlays", name = "LR_F_Hair_006", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_011_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_012_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_Tat_000_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_004_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_006_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_017_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_042_F", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mpxmas_604490_overlays", name = "Hair_014_Fix", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_000", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_003_a", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_003_b", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_003_c", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_003_d", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_003_e", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_005_a", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_005_b", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_005_c", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_005_d", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_005_e", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_006_a", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_006_b", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_006_c", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_006_d", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_006_e", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_013_a", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_013_b", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_013_c", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_013_d", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_013_e", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_014_a", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_014_b", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_014_c", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_014_d", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_014_e", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_long_a", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_long_b", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_long_c", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_long_d", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_long_e", gender = 1, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_001", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_002", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_003", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_004", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_005", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_006", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_007", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_008", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_009", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_010", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_011", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_012", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_013", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_014", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_015", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGBea_M_Hair_000", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGBea_M_Hair_001", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGBus_M_Hair_000", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGBus_M_Hair_001", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGHip_M_Hair_000", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGHip_M_Hair_001", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGInd_M_Hair_000", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_001", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_002", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_003", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_004", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_005", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_006", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_007", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_008", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_009", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_010", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_011", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_012", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_013", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_014", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_015", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGBea_F_Hair_000", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGBea_F_Hair_001", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGBus_F_Hair_000", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGBus_F_Hair_001", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGHip_F_Hair_000", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGHip_F_Hair_001", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGInd_F_Hair_000", gender = 2, zoneId = TattooZone.ZONE_HEAD},
            },
            LEFT_ARM = new List<Tattoo>()
            {
                new Tattoo() { collectionName = "mpairraces_overlays", name = "MP_Airraces_Tattoo_003_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_012_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_016_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_020_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_024_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_025_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_035_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_045_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_053_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_055_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_000", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_010", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_012", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_020", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_021", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_001_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_004_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_007_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_013_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_025_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_029_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_004_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_008_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_015_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_016_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_025_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_027_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphalloween_overlays", name = "MP_Airraces_Tattoo_003_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_003", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_007", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_015", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_016", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_026", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_027", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_028", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_034", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_037", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_039", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_043", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_048", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_004_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_008_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_005_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_027_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_033_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_006_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_018_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_022_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_009_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_020_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_021_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_005_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_016_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_018_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_028_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_031_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_004_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_008_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_014_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_001_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_002_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_008_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_022_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_023_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_035_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_039_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_043_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_001", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_007", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_015", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_005", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_006", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_015", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_031", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_041", gender = 1, zoneId = TattooZone.ZONE_LEFT_ARM},
            },
            RIGHT_ARM = new List<Tattoo>()
            {
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_007_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_014_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_033_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_042_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_046_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_047_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_049_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_054_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_003", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_004", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_008", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_022", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_023", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_026", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_027", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_006_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_012_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_014_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_017_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_018_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_023_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_028_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_002_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_021_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_024_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_001", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_004", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_008", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_010", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_014", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_017", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_018", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_020", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_022", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_023", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_036", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_044", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_045", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_003_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_005_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_006_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_007_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_015_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_003_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_028_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_035_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_004_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_013_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_019_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_010_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_017_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_026_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_030_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_001_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_005_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_023_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_003_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_009_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_010_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_016_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_036_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_038_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_049_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_000", gender = 2, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_002", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_010", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_001", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_003", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_014", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_018", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_027", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_028", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_038", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_047", gender = 1, zoneId = TattooZone.ZONE_RIGHT_ARM},
            },
            LEFT_LEG = new List<Tattoo>()
            {
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_002_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_015_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_027_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_036_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_037_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_044_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_056_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_057_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_001", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_002", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_005_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_007_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_011_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_023_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_009", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_019", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_040", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_007_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_020_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_029_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_000_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_011_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_007_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_013_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_021_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_028_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_031_F", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_009", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_002", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_008", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_021", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_023", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_026", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_032", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_033", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_035", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_037", gender = 1, zoneId = TattooZone.ZONE_LEFT_LEG},
            },
            RIGHT_LEG = new List<Tattoo>()
            {
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_004_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_022_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_028_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_040_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_048_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_F_Tat_014", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_006_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_026_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_030_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_038", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_F_Tat_042", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_017_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_023_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_030_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_001_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_023_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_020_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_005_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_015_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_020_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_025_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_032_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_045_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_047_F", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_F_006", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_007", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_017", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_022", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_039", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_040", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_042", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_F_043", gender = 1, zoneId = TattooZone.ZONE_RIGHT_LEG},
            },
        };

        internal static MaleTattoosCollection MaleTattoos = new MaleTattoosCollection()
        {
            TORSO = new List<Tattoo>()
            {
                new Tattoo() { collectionName = "mpairraces_overlays", name = "MP_Airraces_Tattoo_000_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpairraces_overlays", name = "MP_Airraces_Tattoo_001_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpairraces_overlays", name = "MP_Airraces_Tattoo_002_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpairraces_overlays", name = "MP_Airraces_Tattoo_004_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpairraces_overlays", name = "MP_Airraces_Tattoo_005_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpairraces_overlays", name = "MP_Airraces_Tattoo_006_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpairraces_overlays", name = "MP_Airraces_Tattoo_007_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_000_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_001_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_003_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_005_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_006_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_008_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_010_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_011_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_013_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_017_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_018_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_019_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_021_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_023_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_026_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_029_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_030_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_031_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_032_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_034_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_039_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_041_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_043_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_050_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_052_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_058_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_059_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_060_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_005", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_006", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_009", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_011", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_013", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_015", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_016", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_017", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_018", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_019", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_028", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_000_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_002_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_003_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_005_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_008_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_009_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_010_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_011_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_015_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_016_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_019_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_020_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_021_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_022_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_024_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_026_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_027_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_000_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_001_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_009_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_010_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_012_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_013_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_014_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_017_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_018_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_019_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_020_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_022_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_028_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_029_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphalloween_overlays", name = "MP_Airraces_Tattoo_000_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphalloween_overlays", name = "MP_Airraces_Tattoo_001_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphalloween_overlays", name = "MP_Airraces_Tattoo_002_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphalloween_overlays", name = "MP_Airraces_Tattoo_004_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphalloween_overlays", name = "MP_Airraces_Tattoo_005_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphalloween_overlays", name = "MP_Airraces_Tattoo_006_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphalloween_overlays", name = "MP_Airraces_Tattoo_007_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_000", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_002", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_006", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_011", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_012", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_013", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_024", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_025", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_029", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_030", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_031", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_032", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_033", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_035", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_041", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_046", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_047", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_000_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_001_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_002_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_009_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_010_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_011_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_001_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_002_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_004_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_009_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_010_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_013_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_014_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_021_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_026_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_000_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_008_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_011_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_012_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_016_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_019_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_031_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_032_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_003_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_006_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_007_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_008_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_014_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_015_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_024_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_002_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_012_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_022_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_025_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_027_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_029_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_000_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_002_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_003_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_006_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_007_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_009_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_010_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_013_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_015_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_016_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_017_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_018_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_019_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_021_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_022_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_024_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_025_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_011_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_012_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_014_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_018_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_019_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_024_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_026_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_027_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_029_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_030_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_033_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_034_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_037_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_040_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_041_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_044_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_046_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_048_M", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_003", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_004", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_005", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_008", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_011", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_012", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_013", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_014", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_016", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_017", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_018", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_019", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_004", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_009", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_010", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_011", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_012", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_013", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_016", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_019", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_020", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_024", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_025", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_029", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_030", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_034", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_036", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_044", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_045", gender = 0, zoneId = TattooZone.ZONE_TORSO},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_046", gender = 0, zoneId = TattooZone.ZONE_TORSO},
            },
            HEAD = new List<Tattoo>()
            {
                //new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_Biker_Hair_000_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_Biker_Hair_001_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_Biker_Hair_002_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_Biker_Hair_003_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_Biker_Hair_004_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_Biker_Hair_005_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_Biker_Hair_006_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_009_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_038_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_051_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_007", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_024", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_025", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_029", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Hair_M_000_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Hair_M_001_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_003_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Hair_000_a", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Hair_000_b", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Hair_000_c", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Hair_000_d", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Hair_000_e", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Hair_001_a", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Hair_001_b", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Hair_001_c", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Hair_001_d", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Hair_001_e", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_005", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_021", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Disc_M_Hair_001_a", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Disc_M_Hair_001_b", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Disc_M_Hair_001_c", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Disc_M_Hair_001_d", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Disc_M_Hair_001_e", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mplowrider_overlays", name = "LR_M_Hair_000", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mplowrider_overlays", name = "LR_M_Hair_001", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mplowrider_overlays", name = "LR_M_Hair_002", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mplowrider_overlays", name = "LR_M_Hair_003", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mplowrider2_overlays", name = "LR_M_Hair_004", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mplowrider2_overlays", name = "LR_M_Hair_005", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "mplowrider2_overlays", name = "LR_M_Hair_006", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_011_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_012_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_Tat_000_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_004_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_006_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_017_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_042_M", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_000", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_003_a", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_003_b", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_003_c", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_003_d", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_F_Hair_003_e", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_001_a", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_001_b", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_001_c", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_001_d", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_001_e", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_003_a", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_003_b", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_003_c", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_003_d", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_003_e", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_006_a", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_006_b", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_006_c", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_006_d", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_006_e", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_008_a", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_008_b", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_008_c", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_008_d", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_008_e", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_long_a", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_long_b", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_long_c", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_long_d", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_M_Hair_long_e", gender = 0, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_001", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_002", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_003", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_004", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_005", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_006", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_007", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_008", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_009", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_010", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_011", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_012", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_013", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_014", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_M_Hair_015", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGBea_M_Hair_000", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGBea_M_Hair_001", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGBus_M_Hair_000", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGBus_M_Hair_001", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGHip_M_Hair_000", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGHip_M_Hair_001", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGInd_M_Hair_000", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_001", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_002", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_003", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_004", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_005", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_006", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_007", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_008", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_009", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_010", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_011", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_012", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_013", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_014", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NG_F_Hair_015", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGBea_F_Hair_000", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGBea_F_Hair_001", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGBus_F_Hair_000", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGBus_F_Hair_001", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGHip_F_Hair_000", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGHip_F_Hair_001", gender = 2, zoneId = TattooZone.ZONE_HEAD},
                //new Tattoo() { collectionName = "multiplayer_overlays", name = "NGInd_F_Hair_000", gender = 2, zoneId = TattooZone.ZONE_HEAD},
            },
            LEFT_ARM = new List<Tattoo>()
            {
                new Tattoo() { collectionName = "mpairraces_overlays", name = "MP_Airraces_Tattoo_003_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_012_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_016_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_020_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_024_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_025_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_035_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_045_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_053_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_055_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_000", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_010", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_012", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_020", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_021", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_001_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_004_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_007_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_013_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_025_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_029_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_004_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_008_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_015_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_016_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_025_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_027_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphalloween_overlays", name = "MP_Airraces_Tattoo_003_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_003", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_007", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_015", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_016", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_026", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_027", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_028", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_034", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_037", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_039", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_043", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_048", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_004_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_008_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_005_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_027_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_033_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_006_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_018_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_022_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_009_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_020_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_021_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_005_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_016_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_018_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_028_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_031_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_004_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_008_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_014_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_001_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_002_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_008_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_022_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_023_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_035_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_039_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_043_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_001", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_007", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_015", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_005", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_006", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_015", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_031", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_041", gender = 0, zoneId = TattooZone.ZONE_LEFT_ARM},
            },
            RIGHT_ARM = new List<Tattoo>()
            {
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_007_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_014_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_033_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_042_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_046_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_047_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_049_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_054_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_003", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_004", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_008", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_022", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_023", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_026", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_027", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_006_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_012_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_014_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_017_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_018_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_023_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpchristmas2017_overlays", name = "MP_Christmas2017_Tattoo_028_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_002_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_021_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_024_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_001", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_004", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_008", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_010", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_014", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_017", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_018", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_020", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_022", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_023", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_036", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_044", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_045", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_003_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_005_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_006_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpimportexport_overlays", name = "MP_MP_ImportExport_Tat_007_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_015_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_003_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_028_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_035_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_004_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_013_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_019_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_010_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_017_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_026_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_030_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_001_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_005_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_023_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_003_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_009_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_010_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_016_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_036_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_038_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_049_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_002", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_010", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_000", gender = 2, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_001", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_003", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_014", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_018", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_027", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_028", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_038", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_047", gender = 0, zoneId = TattooZone.ZONE_RIGHT_ARM},
            },
            LEFT_LEG = new List<Tattoo>()
            {
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_002_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_015_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_027_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_036_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_037_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_044_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_056_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_057_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_001", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_002", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_005_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_007_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_011_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_023_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_009", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_019", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_040", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_007_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_020_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_029_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_000_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_011_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_007_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_013_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_021_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_028_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_031_M", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_009", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_002", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_008", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_021", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_023", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_026", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_032", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_033", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_035", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_037", gender = 0, zoneId = TattooZone.ZONE_LEFT_LEG},
            },
            RIGHT_LEG = new List<Tattoo>()
            {
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_004_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_022_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_028_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_040_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpbiker_overlays", name = "MP_MP_Biker_Tat_048_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpchristmas2_overlays", name = "MP_Xmas2_M_Tat_014", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_006_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_026_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpgunrunning_overlays", name = "MP_Gunrunning_Tattoo_030_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_038", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mphipster_overlays", name = "FM_Hip_M_Tat_042", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_017_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mplowrider_overlays", name = "MP_LR_Tat_023_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mplowrider2_overlays", name = "MP_LR_Tat_030_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpluxe_overlays", name = "MP_LUXE_TAT_001_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpluxe2_overlays", name = "MP_LUXE_TAT_023_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpsmuggler_overlays", name = "MP_Smuggler_Tattoo_020_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_005_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_015_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_020_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_025_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_032_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_045_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "mpstunt_overlays", name = "MP_MP_Stunt_tat_047_M", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_Award_M_006", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_007", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_017", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_022", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_039", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_040", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_042", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
                new Tattoo() { collectionName = "multiplayer_overlays", name = "FM_Tat_M_043", gender = 0, zoneId = TattooZone.ZONE_RIGHT_LEG},
            },
        };
        */

    }
}
